(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.4' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     23599,        750]
NotebookOptionsPosition[     21843,        685]
NotebookOutlinePosition[     22196,        701]
CellTagsIndexPosition[     22153,        698]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["The celerite model", "Section",
 CellChangeTimes->{{3.6919366026416492`*^9, 3.691936617787841*^9}, {
  3.6932444266932173`*^9, 3.693244428654402*^9}}],

Cell["\<\
Compute the power spectrum as the Fourier transform of the autocorrelation \
function:\
\>", "Text",
 CellChangeTimes->{{3.691936593653768*^9, 3.691936626091255*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"k", ":=", 
  RowBox[{
   RowBox[{"a", "*", 
    RowBox[{"Exp", "[", 
     RowBox[{
      RowBox[{"-", "c"}], "*", 
      RowBox[{"Abs", "[", "t", "]"}]}], "]"}], "*", 
    RowBox[{"Cos", "[", 
     RowBox[{"d", "*", 
      RowBox[{"Abs", "[", "t", "]"}]}], "]"}]}], "+", 
   RowBox[{"b", "*", 
    RowBox[{"Exp", "[", 
     RowBox[{
      RowBox[{"-", "c"}], "*", 
      RowBox[{"Abs", "[", "t", "]"}]}], "]"}], "*", 
    RowBox[{"Sin", "[", 
     RowBox[{"d", "*", 
      RowBox[{"Abs", "[", "t", "]"}]}], "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"ft", ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"FourierTransform", "[", 
    RowBox[{"k", ",", "t", ",", "w"}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", "ft", "]"}]}], "Input",
 CellChangeTimes->{{3.690479131418994*^9, 3.69047915409188*^9}, {
   3.690560200906397*^9, 3.6905602103523197`*^9}, 3.690560516988418*^9, {
   3.690569670837716*^9, 3.6905696865752363`*^9}, 3.690660583171822*^9, {
   3.690660696778775*^9, 3.690660709135356*^9}, {3.690661691034636*^9, 
   3.6906617280580263`*^9}, {3.690661976780633*^9, 3.690661981147119*^9}, {
   3.6906620850723*^9, 3.690662104159563*^9}, {3.690734604618088*^9, 
   3.6907346131691513`*^9}, {3.6919365159798098`*^9, 3.691936532914249*^9}, {
   3.6919365798545322`*^9, 3.6919365809574327`*^9}, {3.6919369716330013`*^9, 
   3.691936978769003*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox[
    FractionBox["2", "\[Pi]"]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"b", " ", "d", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["c", "2"], "+", 
        SuperscriptBox["d", "2"], "-", 
        SuperscriptBox["w", "2"]}], ")"}]}], "+", 
     RowBox[{"a", " ", "c", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["c", "2"], "+", 
        SuperscriptBox["d", "2"], "+", 
        SuperscriptBox["w", "2"]}], ")"}]}]}], ")"}]}], 
  RowBox[{
   SuperscriptBox["c", "4"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["d", "2"], "-", 
      SuperscriptBox["w", "2"]}], ")"}], "2"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["c", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["d", "2"], "+", 
      SuperscriptBox["w", "2"]}], ")"}]}]}]]], "Output",
 CellChangeTimes->{{3.6904791404105797`*^9, 3.690479159032902*^9}, 
   3.690560216921565*^9, 3.6905605213219976`*^9, 3.690569693305204*^9, 
   3.6906523772264357`*^9, 3.690653335583054*^9, 3.690660584903508*^9, 
   3.690660717735093*^9, 3.690661734607575*^9, 3.6906619859792337`*^9, {
   3.6906620957892303`*^9, 3.6906621104515877`*^9}, 3.690734618261713*^9, 
   3.6919365415889378`*^9, 3.691936633207947*^9, 3.6919369834439163`*^9}]
}, Open  ]],

Cell["\<\
Make sure that the complex formulation of the kernel is equivalent to the \
real version:\
\>", "Text",
 CellChangeTimes->{{3.691936695397788*^9, 3.6919367405756187`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"k2", ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "2"}], "*", 
    RowBox[{"(", 
     RowBox[{"a", "+", 
      RowBox[{"I", "*", "b"}]}], ")"}], "*", 
    RowBox[{"Exp", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"c", "+", 
         RowBox[{"I", "*", "d"}]}], ")"}]}], "*", 
      RowBox[{"Abs", "[", "t", "]"}]}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"1", "/", "2"}], "*", 
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"I", "*", "b"}]}], ")"}], "*", 
    RowBox[{"Exp", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"c", "-", 
         RowBox[{"I", "*", "d"}]}], ")"}]}], "*", 
      RowBox[{"Abs", "[", "t", "]"}]}], "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<k(t) = \>\"", ",", "k2"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"k", "-", "k2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"a", ",", "Reals"}], "]"}], ",", 
     RowBox[{"Element", "[", 
      RowBox[{"b", ",", "Reals"}], "]"}], ",", 
     RowBox[{"Element", "[", 
      RowBox[{"c", ",", "Reals"}], "]"}], ",", 
     RowBox[{"Element", "[", 
      RowBox[{"d", ",", "Reals"}], "]"}]}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.690733298053653*^9, 3.690733330596963*^9}, {
  3.6907333606762743`*^9, 3.6907334148628063`*^9}, {3.690733675183907*^9, 
  3.690733711211554*^9}, {3.6907337646985703`*^9, 3.6907337918327847`*^9}, {
  3.6907340514730997`*^9, 3.690734085216662*^9}, {3.690734166684022*^9, 
  3.690734182659356*^9}, {3.690734656097281*^9, 3.690734671231566*^9}, {
  3.691936643367814*^9, 3.691936685886119*^9}, {3.691936745776649*^9, 
  3.6919367681693287`*^9}, {3.69193682848855*^9, 3.691936831106008*^9}, {
  3.6919369820561123`*^9, 3.6919369996973267`*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"k(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"a", "+", 
       RowBox[{"\[ImaginaryI]", " ", "b"}]}], ")"}], " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "c"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", "d"}]}], ")"}], " ", 
       RowBox[{"Abs", "[", "t", "]"}]}]]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", 
       RowBox[{"\[ImaginaryI]", " ", "b"}]}], ")"}], " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "c"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", "d"}]}], ")"}], " ", 
       RowBox[{"Abs", "[", "t", "]"}]}]]}]}]}],
  SequenceForm[
  "k(t) = ", 
   Rational[1, 2] ($CellContext`a + Complex[0, 1] $CellContext`b) 
    E^((-$CellContext`c + Complex[0, -1] $CellContext`d) Abs[$CellContext`t]) + 
   Rational[1, 2] ($CellContext`a + Complex[0, -1] $CellContext`b) 
    E^((-$CellContext`c + Complex[0, 1] $CellContext`d) Abs[$CellContext`t])],
  
  Editable->False]], "Print",
 CellChangeTimes->{{3.691936664425288*^9, 3.6919366861559277`*^9}, {
   3.691936753921898*^9, 3.6919367685378942`*^9}, 3.6919368316771*^9, 
   3.6919370000853453`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.6907334156932096`*^9, 3.6907337116058617`*^9, {3.69073377053568*^9, 
   3.6907337923637247`*^9}, {3.6907340576589613`*^9, 3.690734085813027*^9}, 
   3.6907341831389637`*^9, {3.690734650233755*^9, 3.690734672775456*^9}, {
   3.6919366580556803`*^9, 3.6919366861711683`*^9}, {3.6919367539358788`*^9, 
   3.6919367685515833`*^9}, 3.691936831691296*^9, 3.691937000131019*^9}]
}, Open  ]],

Cell["Make sure that the full PSD written in the paper is correct:", "Text",
 CellChangeTimes->{{3.691936792959745*^9, 3.691936805998101*^9}, {
  3.6919368656577787`*^9, 3.691936886158876*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"psd", ":=", 
  RowBox[{
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "/", "Pi"}], "]"}], "*", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"a", "*", "c"}], "+", 
         RowBox[{"b", "*", "d"}]}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"c", "^", "2"}], "+", 
         RowBox[{"d", "^", "2"}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"a", "*", "c"}], "-", 
         RowBox[{"b", "*", "d"}]}], ")"}], "*", 
       RowBox[{"w", "^", "2"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"w", "^", "4"}], "+", 
      RowBox[{"2", "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"c", "^", "2"}], "-", 
         RowBox[{"d", "^", "2"}]}], ")"}], "*", 
       RowBox[{"w", "^", "2"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"c", "^", "2"}], "+", 
         RowBox[{"d", "^", "2"}]}], ")"}], "^", "2"}]}], 
     ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<S(w) = \>\"", ",", " ", "psd"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"ft", " ", "-", " ", "psd"}], "]"}]}], "Input",
 CellChangeTimes->{{3.690734589681525*^9, 3.6907346011524057`*^9}, {
  3.6907347076865997`*^9, 3.690734834048904*^9}, {3.690734909296954*^9, 
  3.690734909680855*^9}, {3.6919368121220207`*^9, 3.6919368603927813`*^9}, {
  3.6919370041848583`*^9, 3.691937018497942*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"S(w) = \"\>", "\[InvisibleSpace]", 
   FractionBox[
    RowBox[{
     SqrtBox[
      FractionBox["2", "\[Pi]"]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", " ", "c"}], "+", 
          RowBox[{"b", " ", "d"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["c", "2"], "+", 
          SuperscriptBox["d", "2"]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", " ", "c"}], "-", 
          RowBox[{"b", " ", "d"}]}], ")"}], " ", 
        SuperscriptBox["w", "2"]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["c", "2"], "+", 
        SuperscriptBox["d", "2"]}], ")"}], "2"], "+", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["c", "2"], "-", 
        SuperscriptBox["d", "2"]}], ")"}], " ", 
      SuperscriptBox["w", "2"]}], "+", 
     SuperscriptBox["w", "4"]}]]}],
  SequenceForm[
  "S(w) = ", (2/Pi)^
    Rational[1, 
      2] (($CellContext`a $CellContext`c + $CellContext`b $CellContext`d) \
($CellContext`c^2 + $CellContext`d^2) + ($CellContext`a $CellContext`c - \
$CellContext`b $CellContext`d) $CellContext`w^2)/(($CellContext`c^2 + \
$CellContext`d^2)^2 + 
   2 ($CellContext`c^2 - $CellContext`d^2) $CellContext`w^2 + \
$CellContext`w^4)],
  Editable->False]], "Print",
 CellChangeTimes->{{3.691936842902403*^9, 3.691936861114232*^9}, 
   3.6919370194138727`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.690734834405425*^9, 3.690734910031549*^9, 3.691936778335688*^9, {
   3.691936842912363*^9, 3.691936861128201*^9}, 3.691937019426188*^9}]
}, Open  ]],

Cell["Check some special cases of the PSD:", "Text",
 CellChangeTimes->{{3.6919369053773518`*^9, 3.69193692889517*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"p", ":=", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"2", "*", "Pi"}], "]"}]}], "*", 
   RowBox[{"a", "/", "c"}], "*", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"w", "-", "d"}], ")"}], "/", "c"}], ")"}], "^", "2"}]}], 
       ")"}]}], "+", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"w", "+", "d"}], ")"}], "/", "c"}], ")"}], "^", "2"}]}], 
       ")"}]}]}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<S(w) = \>\"", ",", " ", "p"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"ft", "/.", 
     RowBox[{"{", 
      RowBox[{"b", "\[Rule]", "0"}], "}"}]}], ")"}], "-", "p"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.691937082506669*^9, 3.691937112555191*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"S(w) = \"\>", "\[InvisibleSpace]", 
   FractionBox[
    RowBox[{"a", " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "d"}], "+", "w"}], ")"}], "2"], 
          SuperscriptBox["c", "2"]]}]], "+", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"d", "+", "w"}], ")"}], "2"], 
          SuperscriptBox["c", "2"]]}]]}], ")"}]}], 
    RowBox[{"c", " ", 
     SqrtBox[
      RowBox[{"2", " ", "\[Pi]"}]]}]]}],
  SequenceForm[
  "S(w) = ", $CellContext`a $CellContext`c^(-1) (2 Pi)^
    Rational[-1, 2] ((
      1 + $CellContext`c^(-2) (-$CellContext`d + $CellContext`w)^2)^(-1) + (
      1 + $CellContext`c^(-2) ($CellContext`d + $CellContext`w)^2)^(-1))],
  Editable->False]], "Print",
 CellChangeTimes->{3.691937112915367*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.691937112926182*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"p", ":=", 
  RowBox[{
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "/", "Pi"}], "]"}], "*", 
   RowBox[{"a", "/", 
    RowBox[{"(", 
     RowBox[{"c", " ", "*", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"w", "/", "c"}], ")"}], "^", "2"}]}], ")"}]}], 
     ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<S(w) = \>\"", ",", " ", "p"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"ft", "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"b", "\[Rule]", "0"}], ",", 
       RowBox[{"d", "\[Rule]", "0"}]}], "}"}]}], ")"}], "-", "p"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.6919371617203627`*^9, 3.69193717188063*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"S(w) = \"\>", "\[InvisibleSpace]", 
   FractionBox[
    RowBox[{"a", " ", 
     SqrtBox[
      FractionBox["2", "\[Pi]"]]}], 
    RowBox[{"c", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       FractionBox[
        SuperscriptBox["w", "2"], 
        SuperscriptBox["c", "2"]]}], ")"}]}]]}],
  SequenceForm[
  "S(w) = ", $CellContext`a $CellContext`c^(-1) (2/Pi)^Rational[1, 2]/(
   1 + $CellContext`c^(-2) $CellContext`w^2)],
  Editable->False]], "Print",
 CellChangeTimes->{3.691937172218918*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.691937172228294*^9}]
}, Open  ]],

Cell["Look at the simple harmonic oscillator case:", "Text",
 CellChangeTimes->{{3.6919372137642612`*^9, 3.6919372293058987`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"sho", ":=", 
  RowBox[{
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "/", "Pi"}], "]"}], "*", "S0", "*", 
   RowBox[{
    RowBox[{"w0", "^", "4"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"w", "^", "2"}], "-", 
         RowBox[{"w0", "^", "2"}]}], ")"}], "^", "2"}], "+", 
      RowBox[{
       RowBox[{"w0", "^", "2"}], "*", 
       RowBox[{
        RowBox[{"w", "^", "2"}], "/", 
        RowBox[{"Q", "^", "2"}]}]}]}], ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"ft", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a", "\[Rule]", 
         RowBox[{"S0", "*", "w0", "*", "Q"}]}], ",", 
        RowBox[{"b", "\[Rule]", 
         RowBox[{"S0", "*", "w0", "*", 
          RowBox[{"Q", "/", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{"4", "*", 
              RowBox[{"Q", "^", "2"}]}], "-", "1"}], "]"}]}]}]}], ",", 
        RowBox[{"c", "\[Rule]", 
         RowBox[{"w0", "/", 
          RowBox[{"(", 
           RowBox[{"2", "*", "Q"}], ")"}]}]}], ",", 
        RowBox[{"d", "\[Rule]", 
         RowBox[{"w0", "*", 
          RowBox[{"Sqrt", "[", 
           RowBox[{"1", "-", 
            RowBox[{"1", "/", 
             RowBox[{"(", 
              RowBox[{"4", "*", 
               RowBox[{"Q", "^", "2"}]}], ")"}]}]}], "]"}]}]}]}], "}"}]}], 
     ")"}], "-", "sho"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"w0", ",", "Reals"}], "]"}], ",", 
     RowBox[{"Element", "[", 
      RowBox[{"S0", ",", " ", "Reals"}], "]"}], ",", 
     RowBox[{"Q", ">", "0"}]}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.69193723508505*^9, 3.691937434714612*^9}, {
  3.691938172840958*^9, 3.6919381898177977`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.6919373591617393`*^9, {3.691937401431086*^9, 3.691937435421156*^9}, {
   3.691938174119611*^9, 3.6919381902042007`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"t1", ":=", 
  RowBox[{"ft", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", 
      RowBox[{"S0", "*", "w0", "*", "Q", "*", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"1", "/", 
           RowBox[{"Sqrt", "[", 
            RowBox[{"1", "-", 
             RowBox[{"4", "*", 
              RowBox[{"Q", "^", "2"}]}]}], "]"}]}]}], ")"}], "/", "2"}]}]}], 
     ",", 
     RowBox[{"b", "\[Rule]", "0"}], ",", 
     RowBox[{"c", "\[Rule]", 
      RowBox[{
       RowBox[{"w0", "/", 
        RowBox[{"(", 
         RowBox[{"2", "*", "Q"}], ")"}]}], "*", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"Sqrt", "[", 
          RowBox[{"1", "-", 
           RowBox[{"4", "*", 
            RowBox[{"Q", "^", "2"}]}]}], "]"}]}], ")"}]}]}], ",", 
     RowBox[{"d", "\[Rule]", "0"}]}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"t2", ":=", 
  RowBox[{"ft", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", 
      RowBox[{"S0", "*", "w0", "*", "Q", "*", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"1", "/", 
           RowBox[{"Sqrt", "[", 
            RowBox[{"1", "-", 
             RowBox[{"4", "*", 
              RowBox[{"Q", "^", "2"}]}]}], "]"}]}]}], ")"}], "/", "2"}]}]}], 
     ",", 
     RowBox[{"b", "\[Rule]", "0"}], ",", 
     RowBox[{"c", "\[Rule]", 
      RowBox[{
       RowBox[{"w0", "/", 
        RowBox[{"(", 
         RowBox[{"2", "*", "Q"}], ")"}]}], "*", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"Sqrt", "[", 
          RowBox[{"1", "-", 
           RowBox[{"4", "*", 
            RowBox[{"Q", "^", "2"}]}]}], "]"}]}], ")"}]}]}], ",", 
     RowBox[{"d", "\[Rule]", "0"}]}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"t1", "+", "t2"}], ")"}], "-", "sho"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Element", "[", 
      RowBox[{"w0", ",", "Reals"}], "]"}], ",", 
     RowBox[{"Element", "[", 
      RowBox[{"S0", ",", " ", "Reals"}], "]"}], ",", 
     RowBox[{"Q", ">", "0"}]}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.691937458970121*^9, 3.691937528054791*^9}, {
  3.6919375595891647`*^9, 3.6919375660435467`*^9}, {3.691937646556427*^9, 
  3.691937651812008*^9}, {3.6919379377844543`*^9, 3.691937989711543*^9}, {
  3.691938148739566*^9, 3.691938194369012*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.691937522482616*^9, 3.691937528608686*^9}, 
   3.69193756644981*^9, {3.691937647656296*^9, 3.691937652537051*^9}, {
   3.6919379832632*^9, 3.6919379900561523`*^9}, {3.6919381494407873`*^9, 
   3.691938195820084*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"sho", "/.", 
   RowBox[{"{", 
    RowBox[{"Q", "\[Rule]", 
     RowBox[{"1", "/", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.691940604532209*^9, 3.691940605898543*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox[
    FractionBox["2", "\[Pi]"]], " ", "S0", " ", 
   SuperscriptBox["w0", "4"]}], 
  RowBox[{
   SuperscriptBox["w", "4"], "+", 
   SuperscriptBox["w0", "4"]}]]], "Output",
 CellChangeTimes->{3.6919406062996693`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"sho", "/.", 
   RowBox[{"{", 
    RowBox[{"Q", "\[Rule]", 
     RowBox[{"1", "/", "2"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.691940492942172*^9, 3.691940547234415*^9}, {
  3.691940583508502*^9, 3.691940596346487*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox[
    FractionBox["2", "\[Pi]"]], " ", "S0", " ", 
   SuperscriptBox["w0", "4"]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["w", "2"], "+", 
     SuperscriptBox["w0", "2"]}], ")"}], "2"]]], "Output",
 CellChangeTimes->{{3.691940504754722*^9, 3.691940547855421*^9}, {
  3.6919405859151907`*^9, 3.691940596572817*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"sho", "/.", 
   RowBox[{"{", 
    RowBox[{"Q", "\[Rule]", "Infinity"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.691940706516838*^9, 3.691940721282133*^9}},
 NumberMarks->False],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox[
    FractionBox["2", "\[Pi]"]], " ", "S0", " ", 
   SuperscriptBox["w0", "4"]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["w", "2"], "-", 
     SuperscriptBox["w0", "2"]}], ")"}], "2"]]], "Output",
 CellChangeTimes->{{3.69194070390919*^9, 3.6919407215139933`*^9}}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{720, 851},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
FrontEndVersion->"10.4 for Mac OS X x86 (32-bit, 64-bit Kernel) (April 11, \
2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 156, 2, 64, "Section"],
Cell[739, 26, 176, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[940, 34, 1403, 33, 63, "Input"],
Cell[2346, 69, 1333, 37, 81, "Output"]
}, Open  ]],
Cell[3694, 109, 181, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[3900, 117, 1862, 49, 97, "Input"],
Cell[5765, 168, 1398, 38, 38, "Print"],
Cell[7166, 208, 424, 6, 28, "Output"]
}, Open  ]],
Cell[7605, 217, 193, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[7823, 223, 1536, 46, 80, "Input"],
Cell[9362, 271, 1576, 48, 71, "Print"],
Cell[10941, 321, 189, 3, 28, "Output"]
}, Open  ]],
Cell[11145, 327, 119, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[11289, 332, 1095, 36, 63, "Input"],
Cell[12387, 370, 1033, 31, 82, "Print"],
Cell[13423, 403, 70, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13530, 409, 792, 25, 63, "Input"],
Cell[14325, 436, 552, 17, 78, "Print"],
Cell[14880, 455, 70, 1, 28, "Output"]
}, Open  ]],
Cell[14965, 459, 130, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[15120, 464, 1854, 56, 97, "Input"],
Cell[16977, 522, 173, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17187, 530, 2448, 72, 97, "Input"],
Cell[19638, 604, 267, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19942, 613, 269, 7, 28, "Input"],
Cell[20214, 622, 266, 9, 75, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20517, 636, 283, 7, 28, "Input"],
Cell[20803, 645, 393, 12, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21233, 662, 236, 6, 28, "Input"],
Cell[21472, 670, 343, 11, 81, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

